/*
 * Happy macros for moused filters
 * Makes programming easier?
 */

#include <sys/cdefs.h>
__FBSDID("$FreeBSD$");

#include <sys/types.h>


/* 
 * XXX: Technically, we don't need *any* of these includes, do we?
 * We're just defining macros, afterall. But do we want to require filters.c
 * to have sys/mouse.h and sys/consio.h? Filters using these macros shouldn't
 * need to care about what the underlying structures are.
 */
#include <sys/mouse.h>
#include <sys/consio.h>

/** 
 * Basic Stuff 
 */

#define FILTER(name) static void filter_ ## name(mouse_info_t *delta, int noop) 
#define RESTORE_STATE() (memcpy(delta, &(rodent.state), sizeof(mouse_info_t)))
#define RAW_UPDATE() (ioctl(rodent.cfd, CONS_MOUSECTL, delta))

/**
 * Mouse state accessor/mutator macros
 */

#define MOUSEDATA (delta->u.data)
#define GET_BUTTON_STATE(x) (MOUSEDATA.buttons & (1 << (x - 1)))
#define GET_MOUSE_X() (MOUSEDATA.x)
#define GET_MOUSE_Y() (MOUSEDATA.y)
#define GET_MOUSE_Z() (MOUSEDATA.z)

#define SET_BUTTON_STATE(button) (MOUSEDATA.buttons |= (1 << (button - 1)))
#define CLEAR_BUTTON_STATE(button) (MOUSEDATA.buttons &= ~(1 << (button - 1)))
#define SET_MOUSE_DELTA(axis, delta) MOUSEDATA.axis = delta
#define SET_MOUSE_X(delta) (SET_MOUSE_DELTA(x, delta))
#define SET_MOUSE_Y(delta) (SET_MOUSE_DELTA(y, delta))
#define SET_MOUSE_Z(delta) (SET_MOUSE_DELTA(z, delta))

/**
 * Helpful macros to let filters do magical things 
 */

/* Tell moused to wait for 'ms' milliseconds next time around */
#define NEED_TIMEOUT(ms) rodent.sleeptime.tv_usec = ms * 1000

