/*
 * Happy ps/2 protocol library
 *
 * Ideally, we should provide an api for talking to mice and caring less about
 * what happens underneath.
 */

#include <sys/types.h>

/* XXX: These should disappear when newpsm(4) is done, right? */
#include <sys/mouse.h>
#include <sys/consio.h>
/* Hurray! */

#include <fcntl.h>
#include <stdio.h>
#include <err.h>
#include <errno.h>
#include <assert.h>

#include "ps2_protocol.h"

#define NODATA -1

static int mfd;

int ps2_open(char *dev) {
	int level;
	int hwid = 0;
	mousecommand_t mc;
	char status[3];

	mfd = open(dev, O_RDWR);

	if (mfd == -1) {
		warnx("Failure opening %s: %s", dev, strerror(errno));
		return -1;
	}

	return 0;
}

int ps2_openfd(int fd) {
	mfd = fd;
	return 0;
}

int send_command(int cmd, char data, char *response, int responselen) {
	mousecommand_t mc;
	int ret;

	mc.command = cmd;
	mc.data = data;
	mc.response = response;
	mc.responselen = responselen;

	/* Ensure we can access this portion of memory before going into kernel mode */
	*(response + responselen - 1);

	ret = ioctl(mfd, MOUSE_COMMAND, &mc);

	//warnx("libps2: ioctl: %d", ret);

	return ret;
}

int ps2_reset() {
	return send_command(COMMAND_RESET, NODATA, NULL, 0);
}

int ps2_get_status(char *status) {
	int ret;
	char res[3];

	ret = send_command(COMMAND_STATUS_REQUEST, NODATA, res, 3);

	//warnx("STATUS: %d", ret);
	//warnx("STATUS0: 0x%02x", res[0]);
	//warnx("STATUS1: 0x%02x", res[1]);
	//warnx("STATUS2: 0x%02x", res[2]);
	//warnx("DONE");
	memcpy(status,res,3);

	return ret;
}

int ps2_set_resolution(int resolution) {
	int ret;
	char data;

	/* Valid resolutions are 0-3 */
	assert(resolution >= 0 && resolution <= 3);

	/* Ensure resolution is only 2 bits wide */
	data = resolution & 0x03; 
	ret = send_command(COMMAND_SET_RESOLUTION, data, NULL, 0);

	return ret;
}

int ps2_set_sample_rate(int rate) {
	int ret;
	
	/* Valid sample rates */
	assert(rate == 10 || rate == 20 || rate == 40 || rate == 60
			 || rate == 80 || rate == 100 || rate == 200);

	ret = send_command(COMMAND_SET_SAMPLE_RATE, rate, NULL, 0);

	return ret;
}

int ps2_disable_stream() {
	int ret;

	ret = send_command(COMMAND_DISABLE_REPORTING, NODATA, NULL, 0);
	return ret;
}

int ps2_enable_stream() {
	int ret;

	ret = send_command(COMMAND_ENABLE_REPORTING, NODATA, NULL, 0);
	return ret;
}

int ps2_mousefd() {
	return mfd;
}

int ps2_set_remote_mode() {
	int ret;

	ret = send_command(COMMAND_SET_REMOTE_MODE, NODATA, NULL, 0);
	return ret;
}

int ps2_read_data(char *data, int datalen) {

}

int ps2_ext_command(int command) {
    int c;

    c = (command >> 6) & 0x03;
    if (ps2_set_resolution(c) != 0)
        return 1;

    c = (command >> 4) & 0x03;
    if (ps2_set_resolution(c) != 0)
        return 1;

    c = (command >> 2) & 0x03;
    if (ps2_set_resolution(c) != 0)
        return 1;

    c = (command >> 0) & 0x03;
    if (ps2_set_resolution(c) != 0)
        return 1;
    return 0;
}

int ps2_set_packetsize(int size) {
	int ret;

	ret = ioctl(mfd, MOUSE_SET_PACKETSIZE, &size);
	return ret;
}

int ps2_set_syncmask(unsigned char syncmask) {
	int ret;

	ret = ioctl(mfd, MOUSE_SET_SYNCMASK, &syncmask);
	return ret;
}

