
#include <stdio.h>
#include <sys/select.h>
#include "ps2.h"

static void printbits(char val) {
	int i;
	for (i = 7; i >= 0; i--) {
		printf("%d", (val >> i) & 1);
	}
}

int main(int argc, char **argv) {

	char *dev = "/dev/psm0";
	char status[3];
	char data[12];
	int bytes;
	fd_set fds;

	if (*(argv + 1) != NULL)
		dev = *(argv + 1);

	if (ps2_open(dev) == -1)
		exit(1);

	ps2_reset();
	ps2_disable_stream();

	ps2_ext_command(0);
	ps2_get_status(status);
	printf("Version: %d.%d\n", status[2] & 0x0F, status[0]);

	ps2_ext_command(1);
	ps2_get_status(status);

	printf("0: "), printbits(status[0]), printf("(0x%02x)\n",status[0]);
	printf("1: "), printbits(status[1]), printf("(0x%02x)\n",status[1]);
	printf("2: "), printbits(status[2]), printf("(0x%02x)\n",status[2]);
	//printf("Abs: %d\n", (status[2] & (1 << 7)));
	if ((status[2] & (1 << 7)) == 0) {
		/* Enable Absolute mode, high packet rate, and Wmode ( 0xC1 ) */
		printf("Absolute mode not set, enabling...\n");
		ps2_ext_command(0xc1);
		ps2_set_sample_rate(20); /* 0x14 */
	} else {
		printf("Absolute mode set already?\n");
	}

	ps2_ext_command(1);
	ps2_get_status(status);

	printf("0: "), printbits(status[0]), printf("(0x%02x)\n",status[0]);
	printf("1: "), printbits(status[1]), printf("(0x%02x)\n",status[1]);
	printf("2: "), printbits(status[2]), printf("(0x%02x)\n",status[2]);

	if (status[2] & (1 << 7))
		printf("Absolute mode is set correctly (bit 7, byte 3)\n");
	else
		printf("ABSOLUTE MODE IS OFF!?\n");

	ps2_enable_stream();

	ps2_ext_command(1);
	ps2_get_status(status);

	printf("0: "), printbits(status[0]), printf("(0x%02x)\n",status[0]);
	printf("1: "), printbits(status[1]), printf("(0x%02x)\n",status[1]);
	printf("2: "), printbits(status[2]), printf("(0x%02x)\n",status[2]);

	if (status[2] & (1 << 7))
		printf("Absolute mode is set correctly (bit 7, byte 3)\n");
	else
		printf("2: ABSOLUTE MODE IS OFF!?\n");

	/* Set packetsize to 6 bytes */
	ps2_set_packetsize(6);

	FD_ZERO(&fds);
	FD_SET(ps2_mousefd(), &fds);
	while (1) {
		int x;
		int c;

		c = select(FD_SETSIZE, &fds, NULL, NULL, NULL);
		if (c >= 0) {
			bytes = read(ps2_mousefd(), data, 6);
			if (bytes == 6) {
				printf("long packet\n");
				for (x = 0; x < bytes; x++) {
					printf("   %d: ", x), printbits(data[x]), printf(" (0x%02x)\n", data[x]);
				}
			} else if (bytes == 3) {
				printf("short packet\n");
				//printf("(%d,%d) %d,%d,%d\n", 
						 //data[1] * ((data[0] ^ (1 << 4)) ? -1 : 1),
						 //data[2] * ((data[0] ^ (1 << 5)) ? -1 : 1),
						 //data[0] & (1 << 0),
						 ////data[0] & (1 << 1),
						 //data[0] & (1 << 2));
			}
		}
	}

	return 0;
}
