/*
 * Synaptics touchpad
 *
 */

#include <sys/types.h>
#include <sys/mouse.h>
#include <sys/consio.h>
#include <sys/time.h>

#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <err.h>
#include <errno.h>
#include "../../moused.h"
#include "../../lib/ps2/ps2.h"

#define MOUSE_BUTTON(x) (1 << (x - 1))

#define tv2int(x) ((x.tv_sec * 1000000) + x.tv_usec)

static void printbits(char val) {
	int i;
	for (i = 7; i >= 0; i--) {
		printf("%d", (val >> i) & 1);
	}
}

static void printbits_int(int val) {
	int i;
	for (i = sizeof(val) * 8 - 2; i >= 0; i--) {
		printf("%d", (val >> i) & 1);
	}
}

static void activity(rodent_t *rodent, char *packet);
static void (*logmsg)(int, int, const char *, ...) = NULL;

MOUSED_INIT_FUNC {
	logmsg = rodent->logmsg;
}

MOUSED_PROBE_FUNC {
	char status[3];
	int val;

	/* XXX: Shouldn't moused.c do the open? */
	rodent->mfd = open(rodent->device, O_RDWR);
	if (rodent->mfd == -1)
		logfatal(1, "Unable to open %s", rodent->device);

	ps2_openfd(rodent->mfd);
	ps2_reset();

	val = ps2_get_status(status);
	printf("status: %d\n", val);

	if (val != 0)
		return MODULE_PROBE_FAIL;

	close(rodent->mfd);

	return MODULE_PROBE_SUCCESS;
}

MOUSED_HANDLER_INIT_FUNC {
	char status[3];
	if (rodent->verbose)
		warnx("generic ps/2 init");

	rodent->mfd = open(rodent->device, O_RDWR);
	if (rodent->mfd == -1)
		logfatal(1, "Unable to open %s", rodent->device);

	ps2_openfd(rodent->mfd);
	ps2_reset();

	rodent->mode.packetsize = 3;
	if (NULL == rodent->packet) {
		rodent->packet = malloc(rodent->mode.packetsize);
	}

	ps2_set_packetsize(rodent->mode.packetsize);
	ps2_set_syncmask(0xc0);

	ps2_enable_stream();
}

MOUSED_HANDLER_FUNC {
	int bytes;

	/* XXX: Should this be done in libps2?
	 * ie; bytes = ps2_read_packet(rodent->packet)
	 */
	bytes = read(rodent->mfd, rodent->packet, rodent->mode.packetsize);

	if (bytes < 0)
		return;

	if (bytes == rodent->mode.packetsize)
		activity(rodent, rodent->packet);
	else
		warnx("Invalid packet size: %d", bytes);
}

static void activity(rodent_t *rodent, char *packet) {
	mouse_info_t delta;
	struct timeval uptime;

	delta.operation = MOUSE_ACTION;
	delta.u.data.buttons = 0;
	delta.u.data.x = delta.u.data.y = delta.u.data.z = 0;
	
	delta.u.data.x = packet[1];
	delta.u.data.y = -packet[2];

	if (packet[0] & 0x01)
		delta.u.data.buttons |= MOUSE_BUTTON(1);
	if (packet[0] & 0x02)
		delta.u.data.buttons |= MOUSE_BUTTON(3);
	if (packet[0] & 0x04)
		delta.u.data.buttons |= MOUSE_BUTTON(2);

	rodent->update(&delta);
}

